Package: GCEstim
Type: Package
Title: Regression Coefficients Estimation Using the Generalized Cross
        Entropy
Version: 1.1.0
Date: 2026-06-07
Authors@R: c(
    person("Cabral", "Jorge", email = "jorgecabral@ua.pt", role = c("aut", "cre"),
          comment = c(ORCID = "0000-0001-5721-4550")),
    person("Macedo", "Pedro", role = c("ths")),
    person("Afreixo", "Vera", role = c("ths"))
  )
Description: Estimation and inference using the Generalized Maximum Entropy (GME) and Generalized Cross Entropy (GCE) framework, a flexible method for solving ill-posed inverse problems and parameter estimation under uncertainty (Golan, Judge, and Miller (1996, ISBN:978-0471145925) "Maximum Entropy Econometrics: Robust Estimation with Limited Data"). The package includes routines for generalized cross entropy estimation of linear models including the implementation of a GME-GCE two steps approach. Diagnostic tools, and options to incorporate prior information through support and prior distributions are available (Macedo, Cabral, Afreixo, Macedo and Angelelli (2025) <doi:10.1007/978-3-031-97589-9_21>). In particular, support spaces can be defined by the user or be internally computed based on the ridge trace or on the distribution of standardized regression coefficients. Different optimization methods for the objective function can be used. An adaptation of the normalized entropy aggregation (Macedo and Costa (2019) <doi:10.1007/978-3-030-26036-1_2> "Normalized entropy aggregation for inhomogeneous large-scale data") and a two-stage maximum entropy approach for time series regression (Macedo (2022) <doi:10.1080/03610918.2022.2057540>) are also available. Suitable for applications in econometrics, health, signal processing, and other fields requiring robust estimation under data constraints.
License: GPL-3
Encoding: UTF-8
LazyData: true
Depends: R (>= 3.5.0), zoo
Imports: downlit, data.table, rlang, lbfgs, lbfgsb3c, meboot,
        optimParallel, optimx, rstudioapi, stats, clusterGeneration,
        simstudy, pracma, pathviewr, Rsolnp, bayestestR, ggplot2,
        ggpubr, ggdist, latex2exp, plotly, viridis, hdrcde, shiny,
        miniUI, shinyWidgets, shinydashboardPlus, readxl, DT, magrittr
Suggests: knitr, rmarkdown, kableExtra, NlcOptim
VignetteBuilder: knitr
URL: https://github.com/jorgevazcabral/GCEstim
BugReports: https://github.com/jorgevazcabral/GCEstim/issues
Config/spelling: wordlist: inst/WORDLIST
Config/roxygen2/version: 8.0.0
NeedsCompilation: no
Packaged: 2026-06-07 10:06:45 UTC; jorge
Author: Cabral Jorge [aut, cre] (ORCID:
    <https://orcid.org/0000-0001-5721-4550>),
  Macedo Pedro [ths],
  Afreixo Vera [ths]
Maintainer: Cabral Jorge <jorgecabral@ua.pt>
Repository: CRAN
Date/Publication: 2026-06-07 10:40:02 UTC
Built: R 4.5.2; ; 2026-06-07 11:20:35 UTC; unix
