# Generated by roxygen2: do not edit by hand

S3method(case.names,lmgce)
S3method(coef,cv.lmgce)
S3method(coef,cv.tsbootgce)
S3method(coef,lmgce)
S3method(coef,neagging)
S3method(coef,ridgetrace)
S3method(coef,tsbootgce)
S3method(coefficients,cv.lmgce)
S3method(coefficients,cv.tsbootgce)
S3method(coefficients,lmgce)
S3method(coefficients,neagging)
S3method(coefficients,ridgetrace)
S3method(coefficients,tsbootgce)
S3method(confint,cv.tsbootgce)
S3method(confint,lmgce)
S3method(confint,tsbootgce)
S3method(df.residual,lmgce)
S3method(fitted,lmgce)
S3method(fitted.values,lmgce)
S3method(formula,lmgce)
S3method(model.matrix,lmgce)
S3method(nobs,lmgce)
S3method(plot,cv.lmgce)
S3method(plot,cv.tsbootgce)
S3method(plot,lmgce)
S3method(plot,neagging)
S3method(plot,ridgetrace)
S3method(plot,tsbootgce)
S3method(predict,lmgce)
S3method(print,cv.lmgce)
S3method(print,cv.tsbootgce)
S3method(print,lmgce)
S3method(print,ridgetrace)
S3method(print,summary.lmgce)
S3method(print,tsbootgce)
S3method(resid,lmgce)
S3method(residuals,lmgce)
S3method(summary,lmgce)
S3method(variable.names,lmgce)
S3method(vcov,lmgce)
export(ER.test)
export(NormEnt)
export(accmeasure)
export(changestep)
export(changesupport)
export(cv.dynlmgce)
export(cv.lmgce)
export(cv.tsbootgce)
export(dynlmgce)
export(fngendata)
export(lmgce)
export(lmgceAPP)
export(lmgceAddin)
export(neagging)
export(ridgetrace)
export(scalebackcoef)
export(tsbootgce)
import(miniUI)
import(shiny)
import(shinydashboardPlus)
importFrom(clusterGeneration,rcorrmatrix)
importFrom(downlit,classes_pandoc)
importFrom(downlit,highlight)
importFrom(grDevices,as.graphicsAnnot)
importFrom(graphics,abline)
importFrom(graphics,arrows)
importFrom(graphics,axis)
importFrom(graphics,legend)
importFrom(graphics,lines)
importFrom(graphics,matplot)
importFrom(graphics,mtext)
importFrom(graphics,par)
importFrom(graphics,points)
importFrom(graphics,title)
importFrom(magrittr,"%>%")
importFrom(readxl,read_excel)
importFrom(rlang,.data)
importFrom(shinyWidgets,dropdownButton)
importFrom(shinyWidgets,numericRangeInput)
importFrom(shinyWidgets,pickerInput)
importFrom(shinyWidgets,radioGroupButtons)
importFrom(shinyWidgets,switchInput)
importFrom(shinyWidgets,updatePickerInput)
importFrom(simstudy,genCorData)
importFrom(stats,.checkMFClasses)
importFrom(stats,.getXlevels)
importFrom(stats,as.formula)
importFrom(stats,case.names)
importFrom(stats,coef)
importFrom(stats,coefficients)
importFrom(stats,confint)
importFrom(stats,cycle)
importFrom(stats,delete.response)
importFrom(stats,density)
importFrom(stats,df.residual)
importFrom(stats,fitted)
importFrom(stats,fitted.values)
importFrom(stats,formula)
importFrom(stats,frequency)
importFrom(stats,is.empty.model)
importFrom(stats,is.ts)
importFrom(stats,lag)
importFrom(stats,lm)
importFrom(stats,median)
importFrom(stats,model.frame)
importFrom(stats,model.matrix)
importFrom(stats,model.offset)
importFrom(stats,model.response)
importFrom(stats,na.omit)
importFrom(stats,na.pass)
importFrom(stats,napredict)
importFrom(stats,naprint)
importFrom(stats,naresid)
importFrom(stats,nobs)
importFrom(stats,optim)
importFrom(stats,pchisq)
importFrom(stats,pnorm)
importFrom(stats,predict)
importFrom(stats,printCoefmat)
importFrom(stats,qnorm)
importFrom(stats,quantile)
importFrom(stats,rbinom)
importFrom(stats,relevel)
importFrom(stats,resid)
importFrom(stats,residuals)
importFrom(stats,rnorm)
importFrom(stats,rt)
importFrom(stats,sd)
importFrom(stats,symnum)
importFrom(stats,terms)
importFrom(stats,time)
importFrom(stats,ts)
importFrom(stats,update)
importFrom(stats,var)
importFrom(stats,variable.names)
importFrom(stats,vcov)
importFrom(utils,head)
importFrom(utils,tail)
importFrom(zoo,merge.zoo)
